import pm4py
from pm4py.algo.discovery.dfg import algorithm as dfg_discovery
from pm4py.statistics.start_activities.log import get as start_activities_module
from pm4py.statistics.end_activities.log import get as end_activities_module
from pm4py.util import constants
import pandas as pd

def mine_process(workflow_df, case_name, concept_name, time, transition='frequency'):
    event_log = pm4py.format_dataframe(workflow_df, case_id=case_name, activity_key=concept_name, timestamp_key=time)
    
    if transition=='frequency':
        variant = dfg_discovery.Variants.FREQUENCY
    elif transition=='performance':
        variant = dfg_discovery.Variants.PERFORMANCE
        
    dfg = dfg_discovery.apply(event_log, variant=variant)
    
    grouped_df = event_log.groupby('case:concept:name')
    last_eve_df = grouped_df.last()
    endact_dict = dict(last_eve_df['concept:name'].value_counts())
    
    first_eve_df = grouped_df.first()
    startact_dict = dict(first_eve_df['concept:name'].value_counts())
    
    activities_count = pm4py.get_event_attribute_values(event_log, 'concept:name')
    
    dfg_df = pd.DataFrame([(k[0], k[1], dfg[k]) for k in dfg], columns=['source', 'target', 'weight'])
    
    start_end_mined_df = pd.DataFrame([(k, startact_dict[k], 'start') for k in startact_dict], 
                                     columns=['step', 'frequency', 'start_end']).append(
        pd.DataFrame([(k, endact_dict[k], 'end') for k in endact_dict], 
                                     columns=['step', 'frequency', 'start_end']))
    
    activities_count_df = pd.DataFrame([(k, activities_count[k]) for k in activities_count], columns=['step', 'frequency'])
    
    return dfg_df, start_end_mined_df, activities_count_df
