# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
#%config Completer.use_jedi = False

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
from process_mining.filtering import *

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Read recipe inputs
workflow = dataiku.Dataset("workflow_parsed")
workflow_df = workflow.get_dataframe()

project = dataiku.api_client().get_project(dataiku.default_project_key())
variables = project.get_variables()

events_filters = variables['standard']['filtered_events']
min_time_filter = variables['standard']['min_time_filter']
self_transition = variables['standard']['self_transition']

add_attributes = variables['standard']['add_attributes']
attributes = variables['standard']['attributes']

# column names

case_column = variables['standard']['case']
activity_column = variables['standard']['activity']
timestamp_column = variables['standard']['timestamp']

if add_attributes == "select":
    for attr in attributes:
        workflow_df[attr + ' Attribute'] = workflow_df[attr]

workflow_df.rename(columns={case_column: 'case',
                            activity_column: 'activity',
                            timestamp_column: 'timestamp'},
                   inplace=True)

if add_attributes == "select":
    kept_columns = ['case', 'activity', 'timestamp'] + [attr + ' Attribute' for attr in attributes]
    workflow_df = workflow_df[kept_columns]

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
activity_count = workflow_df.groupby(['activity'])['case'].count().reset_index()

workflow_by_activity = dataiku.Dataset("workflow_by_activity")
workflow_by_activity.write_with_schema(activity_count)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
workflow_df = workflow_df.drop_duplicates()

workflow_df['timestamp'] = pd.to_datetime(workflow_df['timestamp'], infer_datetime_format=True, utc=True)
workflow_df['datetime'] = workflow_df['timestamp'].dt.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
workflow_df['epoch_timestamp'] = pd.to_datetime(workflow_df['timestamp']).astype(int) / 10**9
workflow_df = workflow_df.sort_values(['case', 'epoch_timestamp'])
workflow_df['diff_time'] = workflow_df.groupby(['case'])['epoch_timestamp'].diff().fillna(10000)
workflow_df = workflow_df.drop(['datetime', 'epoch_timestamp'], axis=1)
workflow_df['previous_activity'] = workflow_df.groupby(['case'])['activity'].shift(1)

filtered_workflow = filter_activities(workflow_df, events_filters)
filtered_workflow = filter_time_before(filtered_workflow, min_time_filter)
if self_transition:
    len_workflow = len(filtered_workflow)
    last_len_workflow = 0
    while len_workflow != last_len_workflow:
        filtered_workflow['previous_activity'] = filtered_workflow.groupby(['case'])['activity'].shift(1)
        filtered_workflow = filter_self_transition(filtered_workflow)
        last_len_workflow = len_workflow
        len_workflow = len(filtered_workflow)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
filtered_workflow.drop(columns=['diff_time', 'previous_activity'], inplace=True)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
case_activity_count = filtered_workflow.groupby('case')['activity'].count().reset_index()
unique_activity_case = case_activity_count[case_activity_count['activity']==1]['case']

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
filtered_workflow = filtered_workflow[~filtered_workflow['case'].isin(unique_activity_case)]

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Write recipe outputs
workflow_clean = dataiku.Dataset("workflow_clean")
workflow_clean.write_with_schema(filtered_workflow)